//=============================================================================
// DD_CharacterTraits.js
//=============================================================================

//=============================================================================
 /*:
 * @plugindesc v1.00 Keep track of every main character's traits and emotional well being.
 * @author Harmless
 *
 * @param CharacterNb
 * @desc Number of NPCs that need traits
 * @default 20
 *
 * @param Variable for Character Id
 * @desc ID of the Variable that will store which character is being viewed
 * @default 20
 *
 * @help
 * 
 */
//=============================================================================

var parameters = PluginManager.parameters('DD_CharacterTraits');

var _characterTotal = Number(parameters['CharacterNb'] || 20);
var _characterVar = Number(parameters['Variable for Character Id'] || 55);

var _characterList = [emptyTraitsRow()];

for(var i = 1; i < _characterTotal; i++){
    _characterList[i] = emptyTraitsRow();
}

function emptyTrait(){
    return [0];
}

function emptyTraitsRow(){
    return [emptyTrait(),emptyTrait(),emptyTrait()];
}

//- Traits -//
function getCharacterTraits(id){
    return _characterList[id];
}

function buildCharacterTraits(pos, trait, id){
    var charaid = id || gVar(_characterVar);
    setCharacterTraits(pos, trait, charaid);
}

function buildTrait_A_(trait){
    buildCharacterTraits(0, trait);
}

function buildTrait_B_(trait){
    buildCharacterTraits(1, trait);
}

function buildTrait_C_(trait){
    buildCharacterTraits(2, trait);
}

function setCharacterTraits(pos, trait, id){
    var charaid = id || gVar(_characterVar);
    
    if(typeof trait === "number"){
        var newtrait = [trait];
        trait = newtrait;
    }
    
    _characterList[charaid][pos] = trait;
}

function resetCharacterTraits(){
    setCharacterTraits(0,0);
    setCharacterTraits(1,0);
    setCharacterTraits(2,0);
}

function drawEmptyInfluence(){
    $gameScreen.showPicture(drawInfParam('id'), 'blue_full', 0, drawInfParam('x'), drawInfParam('y'), 100, 100, 255, 0);
}

function drawInfParam(tag){
    switch(tag){
        case 'id':
            return 75;
            break;
        case 'x':
            return 5;
            break;
        case 'y':
            return 4;
            break;
    }
}

//=============================================================================
// Manage Skill Usage on Traits
//=============================================================================

// Core function
function psychoSkillAct(){
    var skillId = gVar(1);
    var traits = _characterList[gVar(_characterVar)];
    var result = psychoSkillConditions(traits,skillId);

    psychoSkillConsequence(result[0], result[1], skillId, false);
}

// Calculates which action's win/loss status
function psychoSkillConditions(traits, skillId){
    var pos = 0;
    var win = false;
    var winReal = false;
    
    for(var x = 0; x < traits.length; x++){
        
        switch(traits[x][0]){
            case 01:
                win = psychoSkillWinId(1,2,skillId);
                break;
            case 02:
                win = psychoSkillWinId(20,21,skillId);
                break;
            case 03:
                win = psychoSkillWinId(1,21,skillId);
                break;
            case 04:
                win = psychoSkillWinId(20,2,skillId);
                break;
                
            case 05:
                win = psychoSkillWinId(23,3,skillId);
                break;
            case 06:
                win = psychoSkillWinId(4,22,skillId);
                break;
            case 07:
                win = psychoSkillWinId(23,22,skillId);
                break;
            case 08:
                win = psychoSkillWinId(4,3,skillId);
                break;
            case 09:
                win = true;
                break;
        }
        
        if(!winReal){
            winReal = win;
            pos = x;
        }
    }
    
    if(winReal == false){
        pos = 0;
        if(skillId > 19){
            skillId -= 19;
        }
        
        if(skillId > 3){
            pos = 1;
        }
    }
    
    return [winReal, pos];
}

// Trigger win situation
function psychoSkillWinId(id_a,id_b, skillId){
    var result = false;
    
    if (skillId == id_a || skillId == id_b){
        result = true;
    }
    
    return result;
}

// Win/Loss consequences
function psychoSkillConsequence(success, pos, skillId, numb){
    if(success){
        editInfluenceOnce(skillId,1);
        traitsFX(2,0);
        sVar(56, 0); // Set SKIP to nothing
    }else{
        editHits(1);
        if(gVar(1) != 0) {
            sVar(56, 0); // Set SKIP to nothing
        }
    }
}

//=============================================================================
// Manage Convo influence
//=============================================================================

function editPoints(amount){
    var current = gVar(31);
    var newVal = current + amount;
    sVar(31,newVal);
    
    if(newVal > 4){
        sVar(31,4);
    }
    
    if(newVal < 0){
        sVar(31,0);
    }
    
    sSwitch(6,true);
    traitsFX(0,97);
}

function editHits(amount){
    var current = gVar(30);
    var newVal = current + amount;
    sVar(30,newVal);
    
    if(newVal > 4){
        sVar(30,4);
    }
    
    if(newVal < -4){
        sVar(30, -4);
    }
    
    sSwitch(30,true);
}

function editInfluence(amount){
    var current = gVar(39);
    var max = gVar(40);
    var newVal = current + amount;
    sVar(39,newVal);
    
    if(newVal > max){
        sVar(39,max);
    }
    
    if(newVal < 0){
        sVar(39,0);
    }
    
    if(current == 0){
        drawEmptyInfluence();
    }
    
    sSwitch(6,true);
}

function resetInfluence(){
    sVar(30,0 - gVar(130));
    sVar(31,4);
    sVar(39,0);
    sVar(40,1);
}