//=============================================================================
// DD_ConnectedMaps.js
//=============================================================================

//=============================================================================
 /*:
 * @plugindesc v0.01 Used to facilitate area traveling on the developper's side (in the style of Zelda:ALTTP)
 * @author Harmless
 *
 * @param Map Width
 * @desc Map's number of X size
 * @default 5
 *
 * @param Map Height
 * @desc Map's number of X size
 * @default 5
 *
 * @param Map Layers
 * @desc Map's number of Z size
 * @default 5
 *
 * @help
 * 
 */
//=============================================================================

var parameters = PluginManager.parameters('DD_ConnectedMaps');

var _mapWidth = Number(parameters['Map Width'] || 5);
var _mapHeight = Number(parameters['Map Height'] || 5);
var _mapLayers = Number(parameters['Map Layers'] || 5);

var _mapPosition = [0,0,0];
var _worldMaps = build3DArray(_mapWidth,_mapHeight,_mapLayers,0);
var _viewedAreas = build3DArray(_mapWidth,_mapHeight,_mapLayers,0);

function setWorldMaps(mapId, val){
    _worldMaps[mapId] = val;
}

function clearWorldMaps(){
    _worldMaps = build3DArray(_mapWidth,_mapHeight,_mapLayers,0);
    _viewedAreas = build3DArray(_mapWidth,_mapHeight,_mapLayers,0);
}

function getMapPosition(){
    return _mapPosition;
}

function setMapPosition(id, x, y, viewArea){
    _mapPosition = [id,y,x];
    
    if(viewArea){
        _viewedAreas[id][y][x] = 1;
    }
}

function moveInMapArray(d){
    var cx = gVar(63);
    var cy = gVar(64);
    var mapPos = getMapPosition();
    
    switch(d){
        case 8:
            // going down
            mapPos[1]++;
            break;
        case 2:
            // going up
            mapPos[1]--;
            break;
        case 6:
            // going left
            mapPos[2]--;
            break;
        case 4:
            // going right
            mapPos[2]++;
            break;
    }
    
    var wID = mapPos[0];
    var wY = mapPos[1];
    var wX = mapPos[2];
    var mapId = _worldMaps[wID][wY][wX];
    setMapPosition(wID,wX,wY,true);
    
    $gamePlayer.reserveTransfer(mapId, cx, cy, 0, 2);
}

function repositionPlayerAfter(d){
    var cx = gVar(63);
    var cy = gVar(64);
    
    switch(d){
        case 8:
            // going down
            cy = 3;
            break;
        case 2:
            // going up
            cy = $dataMap.height - 1;
            break;
        case 6:
            // going left
            cx = $dataMap.width - 2;
            break;
        case 4:
            // going right
            cx = 0;
            break;
    }
    
    sVar(63, cx);
    sVar(64, cy);
    
    positionRealignment();
    
    cx = gVar(63);
    cy = gVar(64);
    
    $gamePlayer.locate(cx, cy);
}

function positionRealignment(){
    var cx = gVar(63);
    var cy = gVar(64);
    var mx = gVar(69);
    var my = gVar(70);
    
    cx += mx*20;
    cy += my*18;
    
    sVar(63, cx);
    sVar(64, cy);
}

function moveInMapFloors(floorDir, transferPlayer, updateMap){
    var cx = $gamePlayer._x;
    var cy = $gamePlayer._y;
    var mapPos = getMapPosition();
    
    var wID = mapPos[0]+floorDir;
    
    if(wID > 5){
        wID = 5;
    }
    
    if(wID < 0){
        wID = 0;
    }
    
    var wY = mapPos[1];
    var wX = mapPos[2];
    var mapId = _worldMaps[wID][wY][wX];
    
    
    setMapPosition(wID,wX,wY,updateMap);
    
    if(transferPlayer){
        $gamePlayer.reserveTransfer(mapId, cx, cy, 0, 2);   
    }
}

function drawHudMapCover(prefix,picId,baseX,baseY){    
    drawMapCover(prefix,picId,baseX,baseY,12,3,null, false);
}

function drawMenuMapCover(prefix,picId,baseX,baseY,flrOverride){
    drawMapCover(prefix,picId,baseX,baseY,60,14, flrOverride, true);
}

function drawMapCover(prefix,picId,baseX,baseY,offsetX,offsetY,flrOverride, hideFlr){
    var picNb;
    var filename;
    var viewData = [];
    var curX;
    var curY;
    
    var mapPos = getMapPosition();
    var posZ = mapPos[0];
    
    if(flrOverride != null){
        if(flrOverride > -1){
            posZ = flrOverride;
        }else{
            sVar(15,posZ);
        }
    }
    
    for(var i = 0; i < 10; i++){
        picNb = i + picId;
        
        if(i % 2 == 0){ // if Even
            if(i == 0){
                viewData[0] = 1;
                viewData[1] = _viewedAreas[posZ][0][0];
                viewData[2] = _viewedAreas[posZ][0][1];
            }else{
                viewData[0] = 1;
                viewData[1] = _viewedAreas[posZ][i/2][0];
                viewData[2] = _viewedAreas[posZ][i/2][1];
            }
            
            if(!hideFlr){
                viewData[0] = 1;
            }
            
            curX = baseX;
        }else{
            viewData[0] = _viewedAreas[posZ][div2(i)][2];
            viewData[1] = _viewedAreas[posZ][div2(i)][3];
            viewData[2] = _viewedAreas[posZ][div2(i)][4];
            
            curX = offsetX + baseX;
        }
        
        filename = String(viewData[0]) + String(viewData[1]) + String(viewData[2]);
        curY = div2(i)*offsetY + baseY;
        
        $gameScreen.showPicture(picNb, prefix+filename, 0, curX, curY, 100, 100, 255, 0);
    }
}

function drawHudMapPos(prefix,picId,baseX,baseY,beeper){
    var mapPos = getMapPosition();
    var posZ = mapPos[0];
    var zMath = (posZ*-3) + 12;
    
    var pic = 'blue';
    
    if(!beeper){
        pic = 'white';
    }
    
    drawMapPos(prefix+pic,picId,baseX + 8,baseY + 2,3,2);
    //$gameScreen.showPicture(picId+1, prefix+'white', 0, baseX, baseY + zMath, 100, 100, 255, 0);
}

function drawMapPos(filename,picId,baseX,baseY,modX,modY){
    var mapPos = getMapPosition();
    var curX = mapPos[2] * modX;
    var curY = mapPos[1] * modY;
    
    $gameScreen.showPicture(picId, filename, 0, baseX + curX, baseY + curY, 100, 100, 255, 0);
}

function exportViewedMap(){
    var flrCompressedInt = 0;
    var flrString = "";
    var roomString = "";
    var mapPos = _mapPosition;
    var mapStr = "";
    
    //floors
    for(var i = 0; i < 5; i++){
        //y
        for(var j = 0; j < 5; j++){
            //x
            for(var k = 0; k < 5; k++){
                sawString = String(_viewedAreas[i][j][k]);
                flrString = flrString + sawString;
            }
        }
        
        flrCompressedInt = parseInt(flrString, 2);
        sVar(81+i, flrCompressedInt);
        flrString = "";
    }
    
    mapStr = String(mapPos[0])+String(mapPos[1])+String(mapPos[2]);
    sVar(86, Number(mapStr));
}

function importViewedMap(){
    var flrCompressedInt = 0;
    var flrBinary = "";
    var flrArray = [];
    var flrString = "";
    var roomString = "";
    var posStr = "";
    
    //floors
    for(var i = 0; i < 5; i++){
        flrCompressedInt = gVar(81+i);
        flrBinary = (+flrCompressedInt).toString(2);
        
        while (flrBinary.length < 25) {
            flrBinary = "0" + flrBinary;
        }
        
        //y
        for(var j = 0; j < 5; j++){
            //x
            for(var k = 0; k < 5; k++){
                _viewedAreas[i][j][k] = Number(flrBinary.charAt(0));
                
                if(flrBinary !== ""){
                    flrBinary = flrBinary.substring(1);
                }
            }
        }
        
        flrBinary = "";
    }
    
    posStr = String(gVar(86));
    
    while (posStr.length < 3) {
        posStr = "0" + posStr;
    }
    
    setMapPosition(Number(posStr.charAt(0)), Number(posStr.charAt(2)), Number(posStr.charAt(1)), true);
}

function div2(val){
    if(val == 0){
        return 0;
    }
    return Math.floor(val/2);
}

function realFlr(val){
    return -(val - 4);
}

function modFlrView(val){
    var currentView = gVar(15);
    var newView = currentView + val;
    var allowSound = true;
    
    if(newView > 4){
        newView = 4;
        allowSound = false;
    }
    
    if(newView < 1){
        newView = 1;
        allowSound = false;
    }
    
    $gameScreen.erasePicture(62);
    sVar(15, newView);

    return allowSound;
}