//=============================================================================
// DD_FaceSet.js
//=============================================================================

/*:
 * @plugindesc v0.05 Calls FaceSet Commen Event by triggering switch
 * @author Harmless
 *
 * @param Picture Id
 * @desc ID of Picture layer used to for the faceset animations, will use the 4 next ones too. (7 = 7 to 11)
 * @default 7
 *
 * @param Loop Switch Id
 * @desc The ID of the switch in which the loop trigger is stored (Buggy)
 * @default 3
 *
 * @param Blink Pause Time
 * @desc The amount of frames between each blink
 * @default 20
 *
 * @param Picture Width
 * @desc The size every img will have
 * @default 214
 *
 * @param Horizontal Margin
 * @desc The amount of space between the img and the window's border
 * @default 24
 *
 * @param Far Horizontal Neg-Margin
 * @desc The amount of space REMOVED between the img and the window's border (When using FarRight & FarLeft)
 * @default 0
 *
 * @param Y Pos
 * @desc The default Y position of the Image
 * @default 239
 *
 * @help
 *
 * Plugin Command:
 *      "FaceSet character1a_3 2 Center 5 -20 true true"
 *      -> Starts bust animation
 *      
 *          # Arg 0 = Image Name prefix,
 *          # Arg 1 = times to loop the talking animation (before switching to blinking)
 *          # Arg 2 = Which side of screen the face is on (Left, Right, Center, FarLeft, FarRight)
 *          # Arg 3 = Modifier to Picture ID (Good for double Facesets)
 *          # Arg 4 = yPos Modifier
 *          # Arg 5 = Is the faceset a Black Silhouettes
 *          # Arg 6 = Is the faceset Flipped Horizontally
 *          # Use X to leave value at previous value ex: "FaceSet X 2 X X"
 *
 *      "FaceSetEnd"   - closes the facesets
 *      "FaceSetEnd 2" - closes the facesets with modifier
 *      -> Closes bust animation
 */

var parameters = PluginManager.parameters('DD_FaceSet');

var pictureId = Number(parameters['Picture Id'] || 7);
var loop_boolId = Number(parameters['Loop Switch Id'] || 3);
var blinkTime = Number(parameters['Blink Pause Time'] || 20);
var pictureWidth = Number(parameters['Picture Width'] || 214);
var xMargin = Number(parameters['Horizontal Margin'] || 24);
var xNegMargin = Number(parameters['Far Horizontal Neg-Margin'] || 0);
var yPos = Number(parameters['Y Pos'] || 239);

var imgX = xMargin;
var imgY = yPos;
var oriantation = "Left";
var windowWidth = 0;
var idMod = 0;
var yMod = 0;
var extraMargin = 10;
var horzSize = 100;

var isNoir = false;

var preSet_img = '';
var preSet_loopNb = 0;

var frameCount; // Which frame to show in the animation cycle
var talkLength; // How many times to loop the talking animation before switching to blinking.

var _Game_Interpreter_pluginCommand =
        Game_Interpreter.prototype.pluginCommand;

Game_Interpreter.prototype.pluginCommand = function(command, args) {
    _Game_Interpreter_pluginCommand.call(this, command, args);
    if (command === 'FaceSet') {
        
        // Resets incomplete animation to frame 1
        if($gameSwitches.value(loop_boolId) == true){
            frameCount = 0;
            showImage(0);
        }
        
        $gameSwitches.setValue(loop_boolId, true);
        
        windowWidth = Graphics.boxWidth;
        
        if(args[4]){
            yMod = Number(args[4]);
        }else{
            yMod = 0;
        }
        
        if(args[3]){
            idMod = Number(args[3]);
        }else{
            idMod = 0;
        }
        
        if(args[2]){
            oriantation = args[2];
        }else{
            oriantation = "Left";
        }
        
        if(args[5]){
            if(String(args[5]) == 'true'){
                isNoir = true;
            }else{
                isNoir = false;
            }
        }
        
        if(args[6]){
            if(String(args[6]) == 'true'){
                horzSize = -100;
            }else{
                horzSize = 100;
            }
        }
        
        if(args[0] == "preSet"){
            args[0] = preSet_img;
        }
        
        if(args[1] == "preSet"){
            args[1] = preSet_loopNb;
        }

        if(args[0] !== "X"){
            var filename = String(args[0]);
            var opacity = 255;
            
            for (i = 0; i < 5; i++) {
                $gameScreen.showPicture(pictureId+idMod+i, filename+'_'+(i+1), 0, getImgX(oriantation, imgX, horzSize), imgY+yMod, horzSize, 100, opacity, 0);
                opacity = 0;
                
                if(isNoir){
                    $gameScreen.tintPicture(pictureId+idMod+i, [-255,-255,-255,0], 1);
                }
            }
        }

        // resets and triggers loop common event
        frameCount = 0;
        talkLength = Number(args[1] || 0);
        $gameSwitches.setValue(loop_boolId, true);
    }
    if (command === 'FaceSetEnd') {
        if(args[0]){
            idMod = Number(args[0]);
        }else{
            idMod = 0;
        }
        
        talkLength = 0;
        frameCount = 0;
        $gameSwitches.setValue(loop_boolId, false);
        isNoir = false;
        horzSize = 100;

        for (i = 0; i < 5; i++) {
            $gameScreen.erasePicture(pictureId+idMod+i);
        }
    }
};

function getImgX(origin, imgX, horzSize) {
    if(origin == "FarLeft"){
        imgX = xNegMargin;
    }
    if(origin == "Left"){
        imgX = xMargin;
    }
    if(origin == "Left2"){
        imgX = xMargin + extraMargin;
    }
    if(origin == "Right"){
        imgX = windowWidth - pictureWidth - xMargin - extraMargin;
    }
    if(origin == "Right2"){
        imgX = windowWidth - pictureWidth - xMargin;
    }
    if(origin == "FarRight"){
        imgX = windowWidth - pictureWidth - xNegMargin;
    }
    if(origin == "Center"){
        imgX = (windowWidth/2) - (pictureWidth/2);
    }
    if(origin == "Center2"){
        imgX = (windowWidth/2) - (pictureWidth/2) + extraMargin;
    }
    
    if(horzSize == -100){
        imgX += pictureWidth;
    }
    
    return imgX;
}

function showImage(picId){
    var opacity = [0,0,0,0,0];
    opacity[picId] = 255;
    
    if($gameSwitches.value(loop_boolId)){
        for (i = 0; i < 5; i++) {
            $gameScreen.movePicture(pictureId + idMod + i, 0, getImgX(oriantation,imgX,horzSize), imgY+yMod, horzSize, 100, opacity[i], 0, 1);
        }
    }
}

function facesetAnimation(){
    // init first image
    if(frameCount == 0){
        showImage(0);
        if(talkLength == 0){
            //frames between each blink
            frameCount = -(blinkTime);
        }
    }
    
    // if counter isn't at frame 0 or waiting, animate!
    if(frameCount > 0){
        
        //if still talk, talk animation
        if(talkLength > 0){
            if(frameCount == 2){
                showImage(2);
            }else{
                showImage(1);
            }
            
        //if not talking, blink animation    
        }else{
            if(frameCount == 2){
                showImage(4);
            }else{
                showImage(3);
            }
        }
    }
}

function facesetLoop(){
    facesetAnimation();
    
    //skip frame 0 when blinking to avoid 'frame 0 init'
    if(frameCount == -1){
        frameCount = 1;
    }
    
    // if counter at max (3), reset counter and ...
    if(frameCount == 3){
        frameCount = -1;
        
        // if still talking, -1 to talk cycle, else: loop blink
        if(talkLength != 0){
            talkLength--;
        }
    }
    
    // +1 to counter, then loop again.
    frameCount++;
}