//=============================================================================
// DD_npcArpgUpgrade.js
//=============================================================================

/*:
 * @plugindesc v0.02 Adds ARPG style features to events allowing combat and more flexible collisions
 * @author Harmless
 *
 * @param Shift Trigger
 * @desc Shift acts as a regular action button while keeping its regular behavior
 * @default false
 *
 * @help
 *
 * Plugin Command:
 *   ARPG xStretch 1               # Makes Character take 2 boxes of size on the x axis
 *   ARPG yStretch 0               # Makes Character take 1 boxes of size on the y axis
 *   ARPG boxShape                 # Makes Character take 3 boxes of size on the x & y axis
 *   ARPG crossShape               # Makes Character take 3 boxes of size on the x & y axis (without counting diagonlly)
 *
 */

(function() {

    var parameters = PluginManager.parameters('DD_npcArpgUpgrade');
    var commandLabel = 'ARPG';
    
    var shiftTrigger = Boolean(parameters['Shift Trigger']);
    
    var _Game_Interpreter_pluginCommand =
            Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === commandLabel) {
            var e = this.character();
        }
    };
    
    // Game_CharacterBase Overriders
    Game_CharacterBase.prototype.initMembers = function() {
        this._x = 0;
        this._y = 0;
        this._realX = 0;
        this._realY = 0;
        this._moveSpeed = 4;
        this._moveFrequency = 6;
        this._opacity = 255;
        this._blendMode = 0;
        this._direction = 2;
        this._pattern = 1;
        this._priorityType = 1;
        this._tileId = 0;
        this._characterName = '';
        this._characterIndex = 0;
        this._isObjectCharacter = false;
        this._walkAnime = true;
        this._stepAnime = false;
        this._directionFix = false;
        this._through = false;
        this._transparent = false;
        this._bushDepth = 0;
        this._animationId = 0;
        this._balloonId = 0;
        this._animationPlaying = false;
        this._balloonPlaying = false;
        this._animationCount = 0;
        this._stopCount = 0;
        this._jumpCount = 0;
        this._jumpPeak = 0;
        this._movementSuccess = true;
        this._pausing = false;
    };
    
    // Check pos (added x+1 and y-1 since sprite x2)
    Game_CharacterBase.prototype.pos = function(x, y) {
        var resultX = this._x === x || this._x === x +1;
        var resultY = this._y === y || this._y === y -1;
        
        var result = resultX && resultY;
        
        return result;
    };
    
    // New Func that extends scope of box colision scan for fake colisions
    Game_CharacterBase.prototype.posPlus = function(x, y) {
        return this.pos(x,y) || this.pos(x-1,y) || this.pos(x-1,y+1) || this.pos(x,y+1);
    };
    
    // - List of funcs edited only to replace pos with posPlus -
        Game_Event.prototype.checkEventTriggerTouch = function(x, y) {
            if (!$gameMap.isEventRunning()) {
                if (this._trigger === 2 && $gamePlayer.posPlus(x, y)) {
                    if (!this.isJumping() && this.isNormalPriority()) {
                        this.start();
                    }
                }
            }
        };

        Game_Map.prototype.eventsXy = function(x, y) {
            return this.events().filter(function(event) {
                return event.posPlus(x, y);
            });
        };

        Game_Player.prototype.isCollided = function(x, y) {
            if (this.isThrough()) {
                return false;
            } else {
                return this.posPlus(x, y) || this._followers.isSomeoneCollided(x, y);
            }
        };
    // - END of list -
    
    Game_Event.prototype.isCollidedWithEvents = function(x, y) {
        var eID = this.eventId();
        var events = $gameMap.eventsXyNt(x, y);
        var newEvents = [];
        
        events.forEach(function(element){
            if(element.eventId() != eID){
                newEvents.push(element);
            }
        });
        
        return newEvents.length > 0;
    };
    
    Game_CharacterBase.prototype.getPausing = function(){
        return this._pausing;
    }
    
    Game_CharacterBase.prototype.setPausing = function(val){
        this._pausing = val;
    }
    
    Game_CharacterBase.prototype.setDirection = function(d) {
        if (!this.isDirectionFixed() && d && !this.getPausing()) {
            this._direction = d;
        }
        this.resetStopCount();
    };
    
    // Check if landscape is passible (added extra 3 boxes to check x+1 AND y-1 since sprite X2)
    Game_CharacterBase.prototype.canPass = function(x, y, d) {
        var x2 = $gameMap.roundXWithDirection(x, d);
        var y2 = $gameMap.roundYWithDirection(y, d);
        var x3 = $gameMap.roundXWithDirection((1 + x), d);
        var y3 = $gameMap.roundYWithDirection((-1 + y), d);
        var x4 = 1 + x;
        var y4 = -1 + y;
        
        var x3inv = $gameMap.roundXWithDirection((-1 + x), d);
        var y3inv = $gameMap.roundYWithDirection((1 + y), d);
        
        if (!$gameMap.isValid(x2, y2) || !$gameMap.isValid(x3, y2) || !$gameMap.isValid(x3, y3) || !$gameMap.isValid(x2, y3)) {
            return false;
        }
        if (this.isThrough() || this.isDebugThrough()) {
            return true;
        }
        
        switch(d){
            case 2:
                if (!this.isMapPassable(x, y, d) || !this.isMapPassable(x4, y, d)){
                    return false;
                }
                break;
            case 4:
                if (!this.isMapPassable(x, y, d) || !this.isMapPassable(x, y4, d)){
                    return false;
                }
                break;
            case 6:
                if (!this.isMapPassable(x4, y, d) || !this.isMapPassable(x4, y4, d)){
                    return false;
                }
                break;
            case 8:
                if (!this.isMapPassable(x, y4, d) || !this.isMapPassable(x4, y4, d)){
                    return false;
                }
                break;
        }
        
        if (this.isCollidedWithCharacters(x2, y2) || this.isCollidedWithCharacters(x3inv, y3inv) || this.isCollidedWithCharacters(x3inv, y2) || this.isCollidedWithCharacters(x2, y3inv)){
            return false;
        }
        
        if(this.getPausing()){
            return false;
        }
        
        return true;
    };
    
    //X offset of character
    Game_CharacterBase.prototype.screenX = function() {
        var tw = $gameMap.tileWidth();
        return Math.round(this.scrolledX() * tw + tw / 2) + 4;
    };

    //Y offset of characters
    Game_CharacterBase.prototype.screenY = function () { 
        var th = $gameMap.tileHeight();
        return Math.round(this.scrolledY() * th + th - this.shiftY() - this.jumpHeight() + (this._walkAnime ? 4 : 4)) + 2; 
    };
    
    //Y offset of PLAYER
    Game_Player.prototype.screenY = function () { 
        var th = $gameMap.tileHeight(); 
        return Math.round(this.scrolledY() * th + th - this.shiftY() - this.jumpHeight() + (this._walkAnime ? 4 : 4)); 
    };
    
    //Screen Center
    Game_Player.prototype.centerX = function() {
        return ((Graphics.width / $gameMap.tileWidth() - 2) / 2.0);
    };

    Game_Player.prototype.centerY = function() {
        return ((Graphics.height / $gameMap.tileHeight() - 2) / 2.0) + 1;
    };
    
    // Game_Player Overriders
    Game_Player.prototype.triggerButtonAction = function() {
        var okBtn = Input.isTriggered('ok');
        var shiftBtn = Input.isTriggered('shift');
        var btnPressed;
        
        if(shiftTrigger){ // global value
            btnPressed = okBtn;
        }else{
            btnPressed = okBtn;
        }
        
        if (btnPressed) {
            if (this.getOnOffVehicle()) {
                return true;
            }
            this.checkEventTriggerHere([0]);
            if ($gameMap.setupStartingEvent()) {
                return true;
            }
            this.checkEventTriggerThere([0,1,2]);
            if ($gameMap.setupStartingEvent()) {
                return true;
            }
        }
        
        return false;
    };
    
    // Allow Event trigger if in mind Item uses
    Game_Player.prototype.triggerAction = function() {
        //default behavior
        if (this.canMove()) {
            if (this.triggerButtonAction()) {
                return true;
            }
            if (this.triggerTouchAction()) {
                return true;
            }
        }
        
        // Stop logic here if event is running
        if ($gameMap.isEventRunning() || $gameMessage.isBusy()) {
            return false;
        }
        
        // Else -> that means that its the item that is false flaging .canMove();
        // So check if shift is pressed and trigger events anyways
        if(Input.isTriggered('shift')){
            if (this.triggerButtonAction()) {
                return true;
            }
        }
        
        return false;
    };
})();
