//=============================================================================
// FA_Functions.js
//=============================================================================

//=============================================================================
 /*:
 * @plugindesc v1.01 JS Sandbox for custom Functions, testing, and other
 * @author Harmless
 *
 * @param console msg
 * @desc what message to display in console at boot
 * @default Hello World!
 *
 * @help
 * JS Sandbox for custom Functions, testing, and other
 */
//=============================================================================

var parameters = PluginManager.parameters('FA_Functions');
var console_msg = String(parameters['console msg']);

initConsoleCall();

// self Managed booleans
var _priveSwitches = [];

for(var i = 1; i < 100; i++){
    _priveSwitches[i] = emptySwitchRow();
}

function emptySwitchRow(){
    return false;
    //return [0,0];
}

function getPriveSwitch(id){
    return _priveSwitches[id];
}

function checkPriveSwitch(id){
    sSwitch(48, getPriveSwitch(id));
}

function setPriveSwitch(id, value){
    _priveSwitches[id] = value;
}

function resetPriveSwitch(){
    for(var j = 1; j < 100; j++){
        setPriveSwitch(j, false);
    }
}

function editInfluenceOnce(switchId, infVal){
    if(getPriveSwitch(switchId) == false){
        editInfluence(infVal);
        winPersuadeTimer();
        setPriveSwitch(switchId, true);
    }
}

/*
 * Game Init console prompt
 */
function initConsoleCall(){
    var bannerWrap = '';

    for (i = 0; i < console_msg.length; i++) {
        bannerWrap += '*';
    }

    console.log(
        '**'+bannerWrap+'**\n'+
        '* '+console_msg+' *\n'+
        '**'+bannerWrap+'**\n\n'
    );
}

/*
 * Turns a Integer into an array containing all digits in inverse order
 */
function intToDigitsArray(x){                           // 432
    var digitNb = x.toString().length;                  // 3
    var digitArray = [0,0,0];                           // Prepopulate for HUD Number system.
    var multiplier;
    
    for(var i = digitNb - 1; i >= 0; i--){
        if(i > 0){
            multiplier = Math.pow(10 , i);            // 2 = (10 * 10) = 100 // 1 = 10
        }else{
            multiplier = 1;                           // 0 = 1
        }
        
        if(i == digitNb - 1){
            digitArray[i] = Math.floor(x/multiplier);   // 432 / 100 = 4.32, floored = 4
        }else if(i < digitNb - 1 && i > 0){
            digitArray[i] = Math.floor(digitArray[i+1]/multiplier);
        }else{
            digitArray[i] = x;
            
            for(var j = digitNb - 1; j > 0; j--){
                digitArray[i] -= (digitArray[j] * Math.pow(10 , j));
            }
        }
    }
    
    return digitArray;
}

// Get variable value shortcut
function gVar(nb){
    return $gameVariables.value(nb);
}

// Set variable to new value shortcut
function sVar(nb, val){
    $gameVariables.setValue(nb, val);
}

// Get switch value shortcut
function gSwitch(nb){
    return $gameSwitches.value(nb);
}

// Set switch to new value shortcut
function sSwitch(nb, val){
    $gameSwitches.setValue(nb, val);
}

// Get bare bones Save State in a Json
function gSaveState(){
    var saveState = {};

    saveState.switches     = $gameSwitches;
    saveState.variables    = $gameVariables;
    saveState.selfSwitches = $gameSelfSwitches;
    saveState.system       = $gameSystem;

    console.log(JsonEx.stringify(saveState));
}

// Set bare bones Save State from a Json
function sSaveState(JSON){
    var saveState = JsonEx.parse(JSON);

    $gameSwitches = saveState.switches;
    $gameVariables = saveState.variables;
    $gameSelfSwitches = saveState.selfSwitches;
    $gameSystem = saveState.system;
}

function gChapterSaveStat(val){
    switch (val) {
        case 2:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,null,false,false,false,true,null,null,null,null,null,false,true,false,null,null,false,true,false,false,false,null,null,null,true,null,true,null,null,null,null,true,false,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,121,null,null,6,null,0,null,null,0,0,0,0,0,1,null,null,null,null,0,0,null,null,null,null,null,null,null,null,null,0,4,null,null,1,93,null,null,null,0,1,null,0,0,2,1,null,null,1,0,null,null,null,null,null,null,null,null,null,null,null,null,null,29,23,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,524416,null,null,0,null,null,0]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true]},{"@c":11,"@a":[null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 3:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,null,null,null,false,true,false,null,null,true,false,false,false,false,null,null,null,true,false,true,null,null,null,null,true,false,null,null,true,null,null,null,null,false,true,false,null,null,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,null,null,null,null,null,null,null,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,null,null,null,null,null,null,null,null,true,true,null,null,null,null,null,null,null,true,true,false,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,null,null,null,null,null,null,null,false]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,20,null,null,6,null,0,null,2,0,0,0,0,0,1,null,0,82,null,0,0,null,null,null,null,null,null,null,null,2,0,4,null,null,2,77,null,null,-1,2,2,null,0,0,1,1,null,null,6,0,null,null,null,null,null,null,null,null,null,null,null,null,null,29,11,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,null,null,4,524416,851968,null,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,2]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true]},{"@c":11,"@a":[null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 4:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,false,null,null,false,true,false,null,null,true,false,false,false,false,null,null,null,true,false,true,null,null,null,null,true,false,null,null,true,null,null,null,null,false,true,false,null,null,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,true,null,null,null,null,null,null,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,null,null,null,null,null,null,null,true,true,true,null,null,null,null,null,null,true,true,true,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,null,false,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,null,null,null,null,null,null,null,false]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,2,null,null,6,null,0,null,3,0,0,0,0,0,1,null,0,83,null,0,0,null,0,null,null,null,null,null,null,1,1,4,null,null,1,77,null,null,-1,2,2,null,0,0,2,1,null,null,10,0,null,null,null,null,null,null,null,null,null,null,null,null,null,18,13,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,null,null,16516,524416,851968,null,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,3]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"42,4,B":true,"68,4,B":true,"68,9,B":true,"68,10,B":true,"69,8,A":true,"77,8,A":true,"77,6,A":true,"77,5,A":true,"69,8,B":true,"66,1,A":true,"84,3,A":true,"84,4,A":true,"84,5,A":true,"84,6,A":true,"84,10,A":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,false,true,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true]},{"@c":11,"@a":[null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 5:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,false,null,null,true,true,false,null,null,true,false,false,false,false,null,null,null,true,false,true,null,null,null,null,true,false,null,null,true,null,null,null,null,false,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,true,null,true,null,null,null,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,false,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,null,true,null,null,null,null,null,true,true,true,true,true,true,null,null,null,true,true,true,true,true,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,null,false,true,null,null,null,true,null,true,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,true,null,null,null,null,true,null,null,true,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,null,null,null,null,null,null,null,false]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,125,null,null,6,null,0,null,3,0,0,0,0,0,1,null,0,85,null,0,0,null,0,null,null,null,null,null,null,1,1,4,1,null,1,93,null,null,-1,3,3,null,0,0,3,1,null,null,10,0,null,null,null,null,null,null,null,null,null,null,null,null,null,9,30,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,null,null,16516,659584,852097,34,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,5]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"42,4,B":true,"68,4,B":true,"68,9,B":true,"68,10,B":true,"69,8,A":true,"77,8,A":true,"77,6,A":true,"77,5,A":true,"69,8,B":true,"66,1,A":true,"84,3,A":true,"84,4,A":true,"84,5,A":true,"84,6,A":true,"84,10,A":true,"84,6,B":true,"87,4,A":true,"84,5,B":true,"106,4,B":true,"106,13,B":true,"106,14,B":true,"138,4,A":true,"138,4,B":true,"136,2,B":true,"135,7,A":true,"134,2,A":true,"135,7,B":true,"44,10,B":true,"6,9,B":true,"111,4,A":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,false,true,null,null,null,null,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true]},{"@c":11,"@a":[null,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 6:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,false,null,null,true,true,false,false,true,false,true,false,false,false,null,null,null,true,false,true,null,null,false,null,true,false,null,null,true,null,null,null,null,false,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,true,null,true,null,null,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,false,true,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,null,true,null,null,null,true,null,true,true,true,true,true,true,null,null,true,true,true,true,true,true,true,false,true,true,true,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,true,false,true,null,null,null,true,null,true,null,null,null,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,true,null,null,null,null,true,null,null,true,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,true,true,null,null,null,null,null,null,null,false]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,129,null,null,6,null,0,null,1,0,0,0,0,0,1,null,0,86,null,0,0,null,0,null,null,null,null,null,null,2,2,4,1,1,1,93,null,null,-1,4,4,null,0,0,4,1,null,null,10,0,null,null,null,null,null,null,null,null,null,null,null,null,null,9,17,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,512,null,16516,659584,852113,16934,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,6]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"42,4,B":true,"68,4,B":true,"68,9,B":true,"68,10,B":true,"69,8,A":true,"77,8,A":true,"77,6,A":true,"77,5,A":true,"69,8,B":true,"66,1,A":true,"84,3,A":true,"84,4,A":true,"84,5,A":true,"84,6,A":true,"84,10,A":true,"84,6,B":true,"87,4,A":true,"84,5,B":true,"106,4,B":true,"106,13,B":true,"106,14,B":true,"138,4,A":true,"138,4,B":true,"136,2,B":true,"135,7,A":true,"134,2,A":true,"135,7,B":true,"44,10,B":true,"6,9,B":true,"111,4,A":true,"123,1,A":true,"123,3,A":true,"123,5,A":true,"123,9,A":true,"123,10,A":true,"123,12,A":true,"123,1,B":true,"125,2,B":true,"125,7,A":true,"125,9,A":true,"127,6,A":true,"128,1,A":true,"128,1,B":true,"123,5,B":true,"129,3,B":true,"132,12,A":true,"123,3,B":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,false,true,null,null,null,null,null,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,null,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true]},{"@c":11,"@a":[null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 7:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,false,null,null,true,true,false,false,true,true,false,false,false,false,null,null,null,true,false,true,null,null,false,null,true,false,null,null,true,null,null,null,null,false,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,true,null,true,false,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,false,true,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,null,true,null,null,null,true,null,true,true,true,true,true,true,null,null,true,true,true,true,true,true,true,false,true,true,true,true,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,true,false,true,null,null,null,true,null,true,null,null,null,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,true,null,null,null,null,true,null,null,true,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,true,true,true,null,null,null,null,null,null,null,null,null,null,true,true,true,null,null,null,null,null,null,null,false]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,130,null,null,6,null,0,null,1,0,0,0,0,0,1,null,0,86,null,0,0,null,0,null,null,null,null,null,null,2,2,4,1,1,1,93,null,null,-1,4,4,null,0,0,1,1,null,null,10,0,null,null,null,null,null,null,null,null,null,null,null,null,null,49,15,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,512,null,16516,659584,852113,16934,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,6]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"42,4,B":true,"68,4,B":true,"68,9,B":true,"68,10,B":true,"69,8,A":true,"77,8,A":true,"77,6,A":true,"77,5,A":true,"69,8,B":true,"66,1,A":true,"84,3,A":true,"84,4,A":true,"84,5,A":true,"84,6,A":true,"84,10,A":true,"84,6,B":true,"87,4,A":true,"84,5,B":true,"106,4,B":true,"106,13,B":true,"106,14,B":true,"138,4,A":true,"138,4,B":true,"136,2,B":true,"135,7,A":true,"134,2,A":true,"135,7,B":true,"44,10,B":true,"6,9,B":true,"111,4,A":true,"123,1,A":true,"123,3,A":true,"123,5,A":true,"123,9,A":true,"123,10,A":true,"123,12,A":true,"123,1,B":true,"125,2,B":true,"125,7,A":true,"125,9,A":true,"127,6,A":true,"128,1,A":true,"128,1,B":true,"123,5,B":true,"129,3,B":true,"132,12,A":true,"123,3,B":true,"120,1,A":true,"120,3,B":true,"121,3,A":true,"119,1,A":true,"84,3,B":true,"84,4,B":true,"77,16,A":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,false,true,null,null,null,null,null,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true]},{"@c":11,"@a":[null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 8:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,false,null,null,true,true,false,false,true,true,false,false,false,false,null,null,null,true,false,true,null,null,false,null,true,false,null,null,true,null,null,null,null,false,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,true,null,true,false,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,false,true,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,null,true,null,null,null,true,null,true,true,true,true,true,true,null,null,true,true,true,true,true,true,true,false,true,true,true,true,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,true,false,true,null,null,null,true,null,true,null,null,null,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,true,null,null,null,null,true,null,null,true,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,null,null,null,null,null,null,null,null,true,true,true,null,null,null,null,null,null,null,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,true,true,true,true,true]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,130,null,null,6,null,0,null,1,0,1,0,0,0,1,null,0,86,null,0,0,null,0,0,null,null,null,null,null,2,0,4,1,1,1,93,null,null,-1,0,1,null,0,0,2,1,null,null,10,0,null,null,null,null,null,null,null,null,null,3,5,null,11,9,47,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,512,null,49284,659584,852113,16934,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,6]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"42,4,B":true,"68,4,B":true,"68,9,B":true,"68,10,B":true,"69,8,A":true,"77,8,A":true,"77,6,A":true,"77,5,A":true,"69,8,B":true,"66,1,A":true,"84,3,A":true,"84,4,A":true,"84,5,A":true,"84,6,A":true,"84,10,A":true,"84,6,B":true,"87,4,A":true,"84,5,B":true,"106,4,B":true,"106,13,B":true,"106,14,B":true,"138,4,A":true,"138,4,B":true,"136,2,B":true,"135,7,A":true,"134,2,A":true,"135,7,B":true,"44,10,B":true,"6,9,B":true,"111,4,A":true,"123,1,A":true,"123,3,A":true,"123,5,A":true,"123,9,A":true,"123,10,A":true,"123,12,A":true,"123,1,B":true,"125,2,B":true,"125,7,A":true,"125,9,A":true,"127,6,A":true,"128,1,A":true,"128,1,B":true,"123,5,B":true,"129,3,B":true,"132,12,A":true,"123,3,B":true,"120,1,A":true,"120,3,B":true,"121,3,A":true,"119,1,A":true,"84,3,B":true,"84,4,B":true,"77,16,A":true,"37,2,A":true,"77,40,A":true,"77,8,B":true,"95,11,A":true,"108,35,A":true,"108,34,A":true,"108,30,A":true,"108,31,A":true,"108,32,A":true,"108,33,A":true,"108,28,A":true,"108,36,A":true,"108,29,A":true,"108,39,A":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,false,true,null,null,null,null,null,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,null,true,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true]},{"@c":11,"@a":[null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
        case 9:
            return '{"switches":{"_data":{"@c":3,"@a":[null,null,null,null,false,false,false,false,false,false,true,null,false,false,null,null,true,true,false,false,true,true,false,false,false,false,null,null,null,true,false,true,null,null,false,null,true,false,null,null,true,null,null,null,null,false,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,false,true,true,true,true,null,true,false,null,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,null,false,true,null,null,null,false,true,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,null,true,null,null,null,true,null,true,true,true,true,true,true,null,null,true,true,true,true,true,true,true,false,true,true,true,true,true,false,false,false,false,false,false,false,false,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,false,true,true,true,true,true,false,true,null,null,null,true,null,true,null,null,null,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true,null,null,null,null,true,null,null,null,null,true,null,null,true,null,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,true,true,true,null,null,null,null,null,true,true,true,null,null,null,null,null,null,null,false,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,true,true,true,true,true]},"@c":2,"@":"Game_Switches"},"variables":{"_data":{"@c":5,"@a":[null,131,null,null,6,null,0,null,1,0,0,0,0,0,1,null,0,86,null,0,0,null,0,0,null,null,null,null,null,2,0,4,1,1,1,93,null,null,-1,0,1,null,0,0,4,1,null,null,10,0,null,null,null,null,null,null,null,null,null,3,5,null,11,18,11,null,0,null,null,0,0,null,null,1,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,16,null,null,4608,null,49284,659584,852113,16934,0,null,null,0,null,null,null,null,null,null,null,null,null,null,null,6]},"@c":4,"@":"Game_Variables"},"selfSwitches":{"_data":{"112,3,A":true,"112,1,A":true,"111,3,A":true,"106,4,A":true,"106,9,A":true,"106,11,A":true,"106,12,A":true,"106,13,A":true,"106,14,A":true,"106,16,A":true,"106,3,A":true,"104,6,A":true,"103,6,A":true,"106,17,A":true,"109,2,A":true,"108,3,A":true,"102,4,B":true,"102,3,A":true,"103,6,B":true,"106,9,B":true,"114,4,A":true,"114,4,B":true,"56,2,A":true,"6,9,A":true,"44,10,A":true,"44,6,A":true,"26,11,A":true,"25,1,A":true,"26,11,B":true,"106,11,B":true,"106,12,B":true,"106,16,B":true,"106,17,B":true,"39,2,A":true,"39,2,B":true,"42,1,A":true,"42,2,B":true,"42,3,A":true,"42,4,A":true,"42,8,A":true,"42,9,A":true,"42,1,B":true,"42,4,B":true,"68,4,B":true,"68,9,B":true,"68,10,B":true,"69,8,A":true,"77,8,A":true,"77,6,A":true,"77,5,A":true,"69,8,B":true,"66,1,A":true,"84,3,A":true,"84,4,A":true,"84,5,A":true,"84,6,A":true,"84,10,A":true,"84,6,B":true,"87,4,A":true,"84,5,B":true,"106,4,B":true,"106,13,B":true,"106,14,B":true,"138,4,A":true,"138,4,B":true,"136,2,B":true,"135,7,A":true,"134,2,A":true,"135,7,B":true,"44,10,B":true,"6,9,B":true,"111,4,A":true,"123,1,A":true,"123,3,A":true,"123,5,A":true,"123,9,A":true,"123,10,A":true,"123,12,A":true,"123,1,B":true,"125,2,B":true,"125,7,A":true,"125,9,A":true,"127,6,A":true,"128,1,A":true,"128,1,B":true,"123,5,B":true,"129,3,B":true,"132,12,A":true,"123,3,B":true,"120,1,A":true,"120,3,B":true,"121,3,A":true,"119,1,A":true,"84,3,B":true,"84,4,B":true,"77,16,A":true,"37,2,A":true,"77,40,A":true,"77,8,B":true,"95,11,A":true,"108,35,A":true,"108,34,A":true,"108,30,A":true,"108,31,A":true,"108,32,A":true,"108,33,A":true,"108,28,A":true,"108,36,A":true,"108,29,A":true,"108,39,A":true,"108,1,A":true,"109,14,A":true,"53,7,A":true,"53,7,B":true,"@c":7},"@c":6,"@":"Game_SelfSwitches"},"system":{"_saveEnabled":true,"_menuEnabled":true,"_encounterEnabled":true,"_formationEnabled":true,"_battleCount":0,"_winCount":0,"_escapeCount":0,"_saveCount":0,"_versionId":0,"_framesOnSave":0,"_bgmOnSave":null,"_bgsOnSave":null,"_windowTone":null,"_battleBgm":null,"_victoryMe":null,"_defeatMe":null,"_savedBgm":null,"_walkingBgm":null,"_wordWrap":true,"_fastForward":true,"_msgFontName":"GameFont","_msgFontSize":13,"_msgFontOutline":4,"_msgSoundEnable":true,"_msgSoundName":"talk_mid","_msgSoundVol":50,"_msgSoundPitch":100,"_msgSoundPitchVar":10,"_msgSoundPan":0,"_msgSoundPanVar":10,"_msgSoundInterval":2,"_msgChoiceMax":6,"_msgWindowPositionX":"auto","_msgWindowPositionY":"auto","_msgWindowAnchorX":0.5,"_msgWindowAnchorY":1,"_EvidenceBookFlags":{"@c":9,"@a":[{"@c":10,"@a":[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,false,true,null,null,null,null,null,true,false,null,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true,true,null,true,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,null,true,true,null,null,null,true]},{"@c":11,"@a":[null,true,true,true,true,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,true,true,true,true]}]},"_messageRows":2,"_hideChoices":{"@c":12,"@a":[]},"_disableChoices":{"@c":13,"@a":[]},"_messageWidth":160,"@c":8,"@":"Game_System"},"@c":1}';
            break;
    }
}

// SetImage on gamePlayer
function playerSetImage(characterName, characterIndex){
    $gamePlayer.setImage(characterName, characterIndex);
}

// build player's characterImg file name using pageId and style
// ex: name_nb_style.png img(1)
function playerImgStyle(characterfile, pageid, characterIndex){
    pageid = String(pageid || 1);
    characterIndex = characterIndex || 0;
    
    var style = 'normal';
    
    if(gSwitch(8)){
        style = 'noir';
    }

    // END of Game Hack
    if(!gSwitch(343)){
        var characterName = characterfile + '_' + pageid + '_' + style + '%(2)';
        playerSetImage(characterName, characterIndex);
    }
}

// Default CharacterImg name (without style)
function playerDefaultCharacterImg(){
    playerImgStyle('buster_regular', 1, 0);
}

// Switch visual style to Noir and reboot characterImg
function switchNoir(val, valAtRoomReboot){
    valAtRoomReboot = valAtRoomReboot || false;
    
    sSwitch(8,val);
    sSwitch(9,valAtRoomReboot);
    playerDefaultCharacterImg();
}

function buildArray(nb,val){
    var result = [];
    
    for(var i = 0; i < nb; i++){
        result[i] = JSON.parse(JSON.stringify(val));
    }
    
    return result;
}

function build2DArray(nbX,nbY,val){
    return buildArray(nbY,buildArray(nbX,val));
}

function build3DArray(nbX,nbY,nbZ,val){
    return buildArray(nbZ,buildArray(nbY,buildArray(nbX,val)));
}

function traitsFX(fxId,imgId){
    $gameVariables.setValue(16,fxId);
    $gameVariables.setValue(17,imgId);
    $gameSwitches.setValue(12, true);
}

function checkAction(action, trait){
    var type = 0;
    var id = 0;
    trait = String(trait || '');
    var traitId = 0;
    
    switch(action){
        
        // POWER
        case 'Your Point Of View':
            type = 1;
            id = 1;
            break;
        case 'pressure':
            type = 1;
            id = 1;
            break;
        case 'Hurt Ego':
            type = 1;
            id = 2;
            break;
        case 'doubt':
            type = 1;
            id = 2;
            break;
        case 'Hold Responsible':
            type = 1;
            id = 3;
            break;
        case 'responsibilize':
            type = 1;
            id = 3;
            break;
        case 'resp':
            type = 1;
            id = 3;
            break;
        case 'Authority':
            type = 1;
            id = 4;
            break;
        case 'credibility':
            type = 1;
            id = 4;
            break;
        case 'cred':
            type = 1;
            id = 4;
            break;
        case 'reframe':
            type = 1;
            id = 5;
            break;
        
        
        // FEELS   
        case 'Their Point Of View':
            type = 2;
            id = 20;
            break;
        case 'pov':
            type = 2;
            id = 20;
            break;
        case 'pointofview':
            type = 2;
            id = 20;
            break;
        case 'Flatter Ego':
            type = 2;
            id = 21;
            break;
        case 'flatter':
            type = 2;
            id = 21;
            break;
        case 'Pardon':
            type = 2;
            id = 22;
            break;
        case 'amnesty':
            type = 2;
            id = 22;
            break;
        case 'Empower':
            type = 2;
            id = 23;
            break;
        case 'relate':
            type = 2;
            id = 23;
            break;
        case 'goodwill':
            type = 2;
            id = 24;
            break;
    }
    
    if(type == 0 && id == 0){
        console.log('ERROR: "'+action+'" isnt listed in checkAction()!!');
    }
    
    if(traitId > 0 && gVar(38) > -1){
        var charaTraits = getCharacterTraits(gVar(8));
        
        if(traitId == charaTraits[gVar(38)][0]){
            sSwitch(46,true);
        }else{
            sSwitch(46,false);
        }
    }else{
        sSwitch(46,true);
    }
    
    if(type == gVar(29) && id == gVar(1)){
        sSwitch(45,true);
    }else{
        sSwitch(45,false);
    }
    
    if(gSwitch(45) && gSwitch(46)){
       sSwitch(47,true); 
    }else{
       sSwitch(47,false);
    }

    if(gVar(1) != 0) {
        sVar(56, gVar(1)); // Set SKIP to skill
    }
}

// Choice Hiding Corrilator
function choiceCorrilator(firstPage,FirstSwitchId){
    var charBranches = [
        gSwitch(FirstSwitchId),
        gSwitch(FirstSwitchId + 1),    
        gSwitch(FirstSwitchId + 2),
        gSwitch(FirstSwitchId + 3),    
        gSwitch(FirstSwitchId + 4)
    ];
    
    if(firstPage){
        // if branch 1 is unlocked, display choice 2
        sSwitch(102, charBranches[0]);
        
        // if branch 2 is unlocked, display choice 3
        sSwitch(103, charBranches[1]);
        
        // if any branch from next page is unlocked, display choice 4
        //if(!charBranches[2] && !charBranches[3] && !charBranches[4]){
        //    sSwitch(104, false);
        //}
    }else{
        // if branch 3 is unlocked, display choice 1
        sSwitch(101, charBranches[2]);
        
        // if branch 3 is unlocked, display choice 2
        sSwitch(102, charBranches[3]);
        
        // if branch 3 is unlocked, display choice 3
        sSwitch(103, charBranches[4]);
    }
}

// Choice Hiding Corrilator (more than 2 pages)
function choiceCorrilatorMulti(pageNb,FirstSwitchId){
    var marginSkip = -6 + (3*pageNb);
    
    if(pageNb>1){
        choiceCorrilator(false,FirstSwitchId+marginSkip);
    }else{
        choiceCorrilator(true,FirstSwitchId);
    }
}

// Telegraph wrong answer
function factsWrongReply(type, factList){
    if(type <= 0){
        type = 1;
    }
    
    for(var i = 0; i < factList.length; i++){
        if(gVar(1) == factList[i]){
            sVar(9, type);
            i = factList.length;
        }
    }
}

// Counts list of switches & returns amount of trues.
function countSwitches(switchId, amount){
    var counting = 0;
    for(var i = 0; i < amount; i++){
        if(gSwitch(switchId + i)){
            counting++;
        }
    }
    
    return counting;
}

// Changes icon depending on if switch True or not
function iconSwitcher(switchId, onIcon, offIcon){
    var i;
    
    if(gSwitch(switchId)){
        i = onIcon;
    }else{
        i = offIcon;
    }
    
    return i;
}

function shutdown_game(){
    window.close();
}

function hud_rm_img(rm_file, opacity, x, y){
    $gameScreen.showPicture(95, 'flr_rm_'+rm_file, 0, x, y, 100, 100, opacity, 0);
}

function usedKeyAlready(){
    sSwitch(191,gSwitch(gVar(43)+60));
}

function setKeySwitch(id){
    sSwitch(id+60,true);
}

function setCharAt(str,index,chr) {
    if(index > str.length-1) return str;
    return str.substr(0,index) + chr + str.substr(index+1);
}

function unlockLockInBin(){
    updateLockBinCrunch(true);
}

function updateLockBinCrunch(isUnlocked){
    var horz = false;
    var oppositeDir = false;
    var dir = $gamePlayer.direction();
    
    switch(dir){
        case 4:
            horz = true;
            oppositeDir = true;
            break;
        case 6:
            horz = true;
            break;
        case 8:
            oppositeDir = true;
            break;
    }
    
    updateLockBinCrunchFull(isUnlocked,horz,oppositeDir);
}

function updateLockBinCrunchFull(isUnlocked,horz,oppositeDir){
    var mapPos = getMapPosition();
    var flr = mapPos[0];
    var y = mapPos[1];
    var x = mapPos[2];
    
    var binCrunch = -1;
    var binBinary = "";
    var strPos = -1;
    var strVal = "0";
    var varId = 91;
    
    if(isUnlocked){
        strVal = "1";
    }
    
    if(oppositeDir){
        if(horz){
            x -= 1;
        }else{
            y -= 1;
        }
        oppositeDir = false;
    }
    
    if(x > -1 && y > -1 && x < 5 && y < 5 && flr > -1 && flr < 6){
        if(horz){
            varId += flr;
        }else{
            varId += 5+flr;
        }
        
        strPos = x+(5*y);
        
        binCrunch = gVar(varId);
        binBinary = (+binCrunch).toString(2);
        
        while (binBinary.length < 20) {
            binBinary = "0" + binBinary;
        }
        
        binBinary = setCharAt(binBinary,strPos,strVal);
        sVar(varId,parseInt( binBinary, 2 ));
    }
}

function quickgetLockBin(){
    var mapPos = getMapPosition();
    var y = mapPos[1];
    var x = mapPos[2];
    
    var horz = false;
    var oppositeDir = false;
    var dir = $gamePlayer.direction();
    
    switch(dir){
        case 4:
            horz = true;
            oppositeDir = true;
            break;
        case 6:
            horz = true;
            break;
        case 8:
            oppositeDir = true;
            break;
    }
    
    return getLockBinCrunch(x,y,horz,oppositeDir);
}

function getLockBinCrunch(x,y,horz,oppositeDir){
    var mapPos = getMapPosition();
    var flr = mapPos[0];
    
    var result = false;
    var binCrunch = -1;
    var binBinary = "";
    var strPos = -1;
    var varId = 91;
    
    
    if(oppositeDir){
        if(horz){
            x -= 1;
        }else{
            y -= 1;
        }
        oppositeDir = false;
    }
    
    if(x > -1 && y > -1 && x < 5 && y < 5 && flr > -1 && flr < 6){
        if(horz){
            varId += flr;
        }else{
            varId += 5+flr;
        }
        
        strPos = x+(5*y);
        
        binCrunch = gVar(varId);
        binBinary = (+binCrunch).toString(2);
        
        while (binBinary.length < 20) {
            binBinary = "0" + binBinary;
        }
        
        if(binBinary.charAt(strPos) == "1"){
            result = true;
        }
    }

    // IF, "No Locks" Mode
    if(gSwitch(32)){
        result = true;
    }
    
    return result;
}

// Interrogation 2.0
//////////////////////

// Configure Intterogation Settings
function interrogationMenuConfig(charaId){

    // Set CharacterId
    sVar(55, charaId);
    
    // Character has been SEEN
    sSwitch(170 + charaId, true);

    // Persuade-Complete Icon?
    sVar(35,iconSwitcher(90 + charaId, 77, 93));

    // Persuade-Complete
    if(isPersuaded()){
        sSwitch(122, false);
    }
}

// Handles Testimony Icon
function hasTestimonyIcon(factId){
    var switchId = 380 + gVar(55);
    var value = gSwitch(switchId) ? factId : 0;
    sVar(54, value);
}

// Flags Results of Questionning
// @return Bool
function questionResult(factId){
    var isRight = false;

    // Lv.1 Right answer?
    if(!isRightFact(factId)){
        
        // Lv.2 Right answer?
        factIdFilter_lv2();
        if(!isRightFact(factId)){

            // Lv.3 Right answer?
            factIdFilter_lv1();
            isRight = isRightFact(factId);
        } else {
            isRight = true;
        }
    } else {
        isRight = true;
    }

    return isRight;
}

// Prompts text if right answer
// @return Bool
function isRightFact(factId){
    var select = gVar(1);
    var factPresentedBefore = gSwitch(380 + gVar(55));
    var rightAnswer = (select == factId);

    if(rightAnswer){
        if(!factPresentedBefore){

            // Set Prompt
            sVar(101, 9);

            // Call Prompt Common Event
            $gameTemp.reserveCommonEvent(21);

            // Suspect has now seen this Fact before
            sSwitch(380 + gVar(55), true);
        }
    }
    
    return rightAnswer;
}

// Water down the Selection Var from Lv.3 values to Lv.2 values...
function factIdFilter_lv2(){
    var select = gVar(1);

    if(select == 65){
        sVar(1, select - 1);
    }
}

// Water down the Selection Var from Lv.2 values to Lv.1 values...
function factIdFilter_lv1(){
    var select = gVar(1);

    if(
        select == 62 || 
        select == 64 || 
        select == 67 || 
        select == 69 || 
        select == 73 || 
        select == 75
    ){
        sVar(1, select - 1);
    }
}

// Returns if current suspect has Persuasion Unlocked
// @return bool
function isInPersuasion(){
    var charaid = gVar(55);
    var isPersuasionUnlocked = gSwitch(160 + charaid);
    return charaid > 0 ? isPersuasionUnlocked : false;
}

// Unlocks Persuasion
function unlockPersuasion(){
    var charaid = gVar(55);
    sSwitch(160 + charaid, true);
}

// Returns if current suspect has been Persuaded
// @return bool
function isPersuaded(){
    var charaId = gVar(55);
    return gSwitch(90 + charaId);
}

function winPersuadeTimer(){
    /*var increaseAmount = 15;
    
    if($gameTimer.getFrames() + increaseAmount * 60 > 60 * 60){
        $gameTimer.setFrames(60 * 60); // 60 seconds
    } else {
        $gameTimer.gainFrames(increaseAmount * 60);
    }*/
}

function animateTitleScreen(picId, posY, opacity, time){
    var picObj = $gameScreen.getPicture(picId);

    $gameScreen.showPicture(picId, picObj.name(), picObj.origin(), picObj.x(), picObj.y(), picObj.scaleX(), picObj.scaleY(), opacity, picObj.blendMode());
    $gameScreen.movePicture(picId, picObj.origin(), picObj.x(), posY, picObj.scaleX(), picObj.scaleY(), opacity, picObj.blendMode(), time);
}

//----------------------------------------

function clearAllPictures(){
    for(var i = 0; i < 101; ++i) {
        $gameScreen.erasePicture(i);
    }
}

//----------------------------------------

function multiPreLoad(pre){
    Galv.CACHE.load('pictures', pre+'_1');
    Galv.CACHE.load('pictures', pre+'_2');
    Galv.CACHE.load('pictures', pre+'_3');
    Galv.CACHE.load('pictures', pre+'_4');
    Galv.CACHE.load('pictures', pre+'_5');
}

function clearGab(){
    SceneManager._scene._gabWindow._gabcurrent = [];
    SceneManager._scene._gabWindow._gabQueue = [];
    SceneManager._scene._gabWindow.clear();
}