//=============================================================================
// FA_Overrider.js
//=============================================================================

//=============================================================================
 /*:
 * @plugindesc v1.00 JS Sandbox for overriding rpgmv core functions
 * @author Harmless
 *
 * @help
 * JS Sandbox for overriding rpgmv core functions
 */
//=============================================================================

var parameters = PluginManager.parameters('FA_Overrider');

// resize window at boot
Yanfly.updateResolution = function() {
    /*var resizeWidth = Yanfly.Param.ScreenWidth - window.innerWidth;
    var resizeHeight = Yanfly.Param.ScreenHeight - window.innerHeight;
    if (!Imported.ScreenResolution) {
        window.moveBy(-1 * resizeWidth / 12, -1 * resizeHeight / 12);
        window.resizeBy(resizeWidth/6, resizeHeight/6);
    };*/
    //window.resizeBy(640 - 816, 576 - 624);
}

Graphics._updateRealScale = function() {
    var h = Math.floor(window.innerWidth / this._width);
    var v = Math.floor(window.innerHeight / this._height);
    this._realScale = Math.min(h, v);
};

// Author : KisaiTenshi
// Clean Image Anti-aliasing
ImageManager.loadBitmap = function(folder, filename, hue, smooth) {
    if (filename) {
        var path = folder + encodeURIComponent(filename) + '.png';
        var bitmap = this.loadNormalBitmap(path, hue || 0);
        //console.log("disabling smoothing for "+path);
        bitmap.smooth = false;
        return bitmap;
    } else {
        return this.loadEmptyBitmap();
    }
};

// Con't remove anti-aliasing
PIXI.settings.SCALE_MODE.DEFAULT = PIXI.SCALE_MODES.NEAREST;

// Remove text outline
var _Window_Base_ResetFontSettings = Window_Base.prototype.resetFontSettings;
Window_Base.prototype.resetFontSettings = function() {
    _Window_Base_ResetFontSettings.call( this );
    this.contents.outlineWidth = 0;
};

// repostion icons
Window_Base.prototype.processDrawIcon = function(iconIndex, textState) {
    this.drawIcon(iconIndex, textState.x + 2, textState.y);
    textState.x += Window_Base._iconWidth + 4;
};

// Fix text height
Window_Base.prototype.calcTextHeight = function(textState, all) {
    var lastFontSize = this.contents.fontSize;
    var textHeight = 0;
    var lines = textState.text.slice(textState.index).split('\n');
    var maxLines = all ? lines.length : 1;

    for (var i = 0; i < maxLines; i++) {
        var maxFontSize = this.contents.fontSize;
        var regExp = /\x1b[\{\}]/g;
        for (;;) {
            var array = regExp.exec(lines[i]);
            if (array) {
                if (array[0] === '\x1b{') {
                    this.makeFontBigger();
                }
                if (array[0] === '\x1b}') {
                    this.makeFontSmaller();
                }
                if (maxFontSize < this.contents.fontSize) {
                    maxFontSize = this.contents.fontSize;
                }
            } else {
                break;
            }
        }
        textHeight += maxFontSize + 4;
    }

    this.contents.fontSize = lastFontSize;
    return textHeight;
};

Window_Base.prototype.translucentOpacity = function() {
    return 70;
};

/*************************************
 *       GameBoy-Like Controls       *
 *************************************
 * D-Pad    =   Arrows, WASD, numpad *
 * A        =   Space, Enter, Z      *
 * B        =   Shift, C             *
 * Start    =   Esc, Ins, X, num0    *
 * Select   =   pageup               *
 *************************************/

// Input keyMapper - Button to Key Mapper
Input.keyMapper = {
    9: 'tab',       // tab
    13: 'ok',       // enter
    16: 'shift',    // shift
    17: 'control',  // control
    18: 'control',  // alt
    27: 'escape',   // escape
    32: 'ok',       // space
    33: 'pageup',   // pageup
    //34: 'pagedown', // pagedown
    37: 'left',     // left arrow
    38: 'up',       // up arrow
    39: 'right',    // right arrow
    40: 'down',     // down arrow
    45: 'escape',   // insert
    
    //81: 'pageup',   // Q
    87: 'up',       //'pagedown', // W
    65: 'left',     // A
    83: 'down',     // S
    68: 'right',    // D
    
    88: 'escape',   // X
    90: 'ok',       // Z
    67: 'shift',    // C
    86: 'pageup',   // V
    96: 'escape',   // numpad 0
    98: 'down',     // numpad 2
    100: 'left',    // numpad 4
    102: 'right',   // numpad 6
    104: 'up',      // numpad 8
    120: 'debug'    // F9
};

//Save Data ---------------------------------------------------
DataManager.makeSavefileInfo = function() {
    var factStr = String($gameVariables.value(87));
    if(factStr.length < 2){
        factStr = "0" + factStr;
    }
    
    var info = {};
    
    info.globalId   = this._globalId;
    info.title      = $dataSystem.gameTitle;
    info.characters = $gameParty.charactersForSavefile();
    info.faces      = $gameParty.facesForSavefile();
    info.playtime   = $gameSystem.playtimeText();
    info.timestamp  = Date.now();
    
    info.facts      = factStr;
    info.level      = $gameVariables.value(32) + $gameVariables.value(33) + $gameVariables.value(130);
    
    return info;
};

//Save Window -------------------------------------------------
// Max Number of saves
DataManager.maxSavefiles = function() {
    return 3;
};

Window_Base.prototype.drawCharacterCustom = function(characterName, characterIndex, x, y) {
    var bitmap = ImageManager.loadCharacter(characterName);
    var big = ImageManager.isBigCharacter(characterName);
    var pw = bitmap.width / (big ? 2 : 8);
    var ph = bitmap.height / (big ? 4 : 8);
    var n = characterIndex;
    var sx = (n % 4 * 2) * pw;
    var sy = (Math.floor(n / 4) * 4) * ph;
    this.contents.blt(bitmap, sx, sy, pw, ph, x - pw / 2, y - ph);
};

Window_SavefileList.prototype.itemHeight = function() {
    var innerHeight = this.height - this.padding * 2;
    return Math.floor(innerHeight / 3);
};

// checks if is file empty or not and sends it to drawFileId func.
Window_SavefileList.prototype.drawItem = function(index) {
    var id = index + 1;
    var valid = DataManager.isThisGameFile(id);
    var info = DataManager.loadSavefileInfo(id);
    var rect = this.itemRectForText(index);
    this.resetTextColor();
    if (this._mode === 'load') {
        this.changePaintOpacity(valid);
    }
    this.drawFileId(id, rect.x, rect.y, valid);
    
    this.changePaintOpacity(valid);
    this.drawContents(info, rect, valid);
    this.changePaintOpacity(true);
};

// File label
Window_SavefileList.prototype.drawFileId = function(id, x, y, valid) {
    this.changeTextColor(this.textColor(4));
    this.drawText('Detective ' + id, x + 20, y - 2, 160);
    this.changeTextColor(this.textColor(0));
};

// Name label
Window_SavefileList.prototype.drawContents = function(info, rect, valid) {
    var bottom = rect.y + rect.height;
    var lineHeight = this.lineHeight();
    
    if(valid){
        this.drawCharacterCustom("buster_regular_1_normal%(2)", 0, rect.x + 8, rect.y + 16);
        
        this.drawPlaytime(info, rect.x + 20, rect.y + 10, rect.width);
        this.drawIcon(91, rect.x + 88, rect.y + 7);
        this.drawText(info.level, rect.x + 104, rect.y + 10, 28);
        this.drawIcon(73, rect.x + 112, rect.y + 7);
        this.changeTextColor(this.textColor(5));
        this.drawText(info.facts, rect.x + 112, rect.y + 10, 28, 'right');
        this.changeTextColor(this.textColor(0));
    }else{
        this.drawCharacterCustom("buster_regular_1_normal%(2)", 6, rect.x + 8, rect.y + 16);
        this.changeTextColor(this.textColor(2));
        this.drawText('Off Commission...', rect.x + 20, rect.y + 10, 160);
        this.changeTextColor(this.textColor(0));
    }
};

Window_SavefileList.prototype.drawPlaytime = function(info, x, y, width) {
    if (info.playtime) {
        this.drawText(info.playtime, x, y, width);
    }
};

//Fix width not found bug
Sprite.prototype._onBitmapLoad = function() {
    if(this._bitmap !== null){
        if (this._frame.width === 0 && this._frame.height === 0) {
            this._frame.width = this._bitmap.width;
            this._frame.height = this._bitmap.height;
        }   
    }
    this._refresh();
};

// Remove bugfix from Pictures, to prevent unfully loaded images
Sprite_Picture.prototype._onBitmapLoad = function(bitmapLoaded) {
    if(bitmapLoaded === this._bitmap){
        if (this._refreshFrame && this._bitmap) {
            this._refreshFrame = false;
            //console.log(this._frame.width, this._frame.height, '-', this._bitmap.width, this._bitmap.height);
            //console.log(this);
            this._frame.width = this._bitmap.width;
            this._frame.height = this._bitmap.height;
            
        }
    }

    this._refresh();
};

// Slower run / Faster walk
Game_CharacterBase.prototype.realMoveSpeed = function() {
    return this._moveSpeed + 1 + (this.isDashing() ? 0.70 : 0);
};

// No running while event moves player
Game_Player.prototype.updateDashing = function() {
    if (this.isMoving()) {
        return;
    }
    if (this.canMove() && !this.isInVehicle() && !$gameMap.isDashDisabled()) {
        this._dashing = this.isDashButtonPressed(); //|| $gameTemp.isDestinationValid();
    } else {
        this._dashing = false;
    }
};

//makes animation faster
Game_Player.prototype.animationWait = function() {
    return this.realMoveSpeed() + 3;
};

Game_CharacterBase.prototype.animationWait = function() {
    return (this.realMoveSpeed() + 7);
};

//close btn trigger when window opens
Window_Base.prototype.open = function() {
    if (!this.isOpen()) {
        this._opening = true;
        $gameSwitches.setValue(21, false);
    }
    this._closing = false;
};

//open btn trigger when window closes
Window_Base.prototype.updateClose = function() {
    if (this._closing) {
        this.openness -= 32;
        if (this.isClosed()) {
            this._closing = false;
            $gameSwitches.setValue(21, true);
        }
    }
};

//Fix width of option
Window_Options.prototype.windowWidth = function() {
    return 128;
};

Window_DebugRange.prototype.windowWidth = function() {
    return 50;
};

Window_DebugRange.prototype.drawItem = function(index) {
    var rect = this.itemRectForText(index);
    var start;
    var text;
    if (index < this._maxSwitches) {
        start = index * 10 + 1;
        text = 's';
    } else {
        start = (index - this._maxSwitches) * 10 + 1;
        text = 'v';
    }
    var end = start + 9;
    text += start.padZero(3);
    this.drawText(text, rect.x, rect.y, rect.width);
};

Window_DebugEdit.prototype.drawItem = function(index) {
    var dataId = this._topId + index;
    var idText = dataId.padZero(2) + ' ';
    var idWidth = this.textWidth(idText);
    var statusWidth = this.textWidth('-00000000');
    var name = this.itemName(dataId);
    var status = this.itemStatus(dataId);
    var rect = this.itemRectForText(index);
    this.resetTextColor();
    this.drawText(idText, rect.x, rect.y, rect.width);
    rect.x += idWidth;
    rect.width -= idWidth + statusWidth;
    this.drawText(name, rect.x, rect.y, rect.width);
    this.drawText(status, rect.x + rect.width, rect.y, statusWidth, 'right');
};

Window_DebugEdit.prototype.initialize = function(x, y, width) {
    var height = this.fittingHeight(5);
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this._mode = 'switch';
    this._topId = 1;
    this.refresh();
};

Window_DebugEdit.prototype.itemName = function(dataId) {
    var result;
    
    if (this._mode === 'switch') {
        result = $dataSystem.switches[dataId];
    } else {
        result = $dataSystem.variables[dataId];
    }
    
    if(result != null){
        if(result.length > 7){
            result = result.substring(0,7);
        }
    }
    
    return result;
};

Window_DebugEdit.prototype.itemStatus = function(dataId) {
    if (this._mode === 'switch') {
        return $gameSwitches.value(dataId) ? 'ON' : 'OFF';
    } else {
        return String($gameVariables.value(dataId));
    }
};

// Remove addGeneralOptions
Window_Options.prototype.addGeneralOptions = function(){};

Window_Options.prototype.volumeOffset = function() {
    return 5;
};

// Start Music At Half Volume (BROKEN)
/*AudioManager._bgmVolume      = 50;
AudioManager._bgsVolume      = 50;
AudioManager._meVolume       = 50;
AudioManager._seVolume       = 50;*/

// Turn Shop into Debug menu -START-
Scene_Shop.prototype.create = function() {
    Scene_MenuBase.prototype.create.call(this);
    this.createBuyWindow();
};

Scene_Shop.prototype.createBuyWindow = function() {
    var wy = 0;
    var wh = Graphics.boxHeight;
    this._buyWindow = new Window_ShopBuy(0, wy, wh, this._goods);
    this._buyWindow.setHandler('ok',     this.onBuyOk.bind(this));
    this._buyWindow.setHandler('cancel', this.popScene.bind(this));
    this.addWindow(this._buyWindow);

    this._buyWindow.setMoney(1);
    this._buyWindow.show();
    this._buyWindow.activate();
};

Scene_Shop.prototype.onBuyOk = function() {
    $gameParty.gainItem(this._buyWindow.item(), 1, false);
    this._buyWindow.refresh();
    this._buyWindow.activate();
};

Window_ShopBuy.prototype.windowWidth = function() {
    return 160;
};

Window_ShopBuy.prototype.drawItem = function(index) {
    var item = this._data[index];
    var rect = this.itemRect(index);
    rect.width -= this.textPadding();
    this.changePaintOpacity($gameParty.hasItem(item));
    this.drawItemName(item, rect.x, rect.y, rect.width);
    this.changePaintOpacity(true);
};
// Turn Shop into Debug menu -END-


// Prevents color from reset in texts //
    Window_Message.prototype.initialize = function() {
        var width = this.windowWidth();
        var height = this.windowHeight();
        var x = (Graphics.boxWidth - width) / 2;
        Window_Base.prototype.initialize.call(this, x, 0, width, height);
        this.openness = 0;
        this._normalColor;
        this.initMembers();
        this.createSubWindows();
        this.updatePlacement();
    };

    Window_Message.prototype.normalColor = function() {
        if(!this._normalColor){
            this._normalColor = this.textColor(0);
        }

        return this._normalColor;
    };

    Window_Message.prototype.setNormalColor = function(color) {
        this._normalColor = color;
    };

    Window_Message.prototype.changeTextColor = function(color) {
        if (this._sfont) {
            this.contents.sfont = this._sfont;
            this._sfont = null;
        }

        this.setNormalColor(color);
        this.contents.textColor = color;
    };

    Window_Message.prototype.newPage = function(textState) {
        this._sfont = VictorEngine.SFont.getSFont(this.normalColor);

        // yanfly msg plugin 2
        this._soundCount = 0;

        // yanfly msg plugin 1
        this.adjustWindowSettings();

        // original
        this.contents.clear();
        this.resetFontSettings();
        this.clearFlags();
        this.loadMessageFace();
        textState.x = this.newLineX();
        textState.y = 0;
        textState.left = this.newLineX();
        textState.height = this.calcTextHeight(textState, false);
    };
// END of color reset prevents //

// So you can have 999 of money
Game_Party.prototype.maxItems = function(item) {
    return 999;
};

//Get access to Game_Picture object
Game_Screen.prototype.getPicture = function(pictureId) {
    var realPictureId = this.realPictureId(pictureId);
    return this._pictures[realPictureId];
};

Game_Player.prototype.refresh = function() {
    var actor = $gameParty.leader();
    
    if(!gSwitch(18)){
        var characterName = actor ? actor.characterName() : '';
        var characterIndex = actor ? actor.characterIndex() : 0;
        this.setImage(characterName, characterIndex);
    }
    
    this._followers.refresh();
};

//Text Scroll Speed /3
Game_Message.prototype.setScroll = function(speed, noFast) {
    this._scrollMode = true;
    this._scrollSpeed = speed/2.5;
    this._scrollNoFast = noFast;
};

// Override SRD_PictureCHoice choice = width, height, y
// width
Window_PictureChoiceList.prototype.windowWidth = function() {
	return $gameMessage.pictureChoiceSize[0] - 16;
};

// height
Window_PictureChoiceList.prototype.windowHeight = function() {
	if(!this._messageWindow.isClosed()) {
		return $gameMessage.pictureChoiceSize[1] - this._messageWindow.height - 10 - 86;
	}
	return $gameMessage.pictureChoiceSize[1] - 86;
};

// y
Window_PictureChoiceList.prototype.updatePlacement = function() {
	this.width = this.windowWidth();
	this.height = this.windowHeight();
	this.x = (Graphics.boxWidth - this.width - 6) / 2;
	this.y = (Graphics.boxHeight - this.height + 94) / 2;
};

// Fixing HUD
// Items Icon spacing
Galv.CI.Window_Selectable_drawItemName = function(item, x, y, width) {
    width = width || 312;
    if (item) {
        var iconBoxWidth = this.lineHeight();
        var padding = (iconBoxWidth - Window_Base._iconWidth) / 2;
        this.resetTextColor();
        this.drawIcon(item.iconIndex, x + padding + 1, y + padding);
        this.drawText(item.name, x + iconBoxWidth + 4, y, width - iconBoxWidth - 4);
    }
};

// Music Blocker feature
AudioManager.playBgm = function(bgm, pos) {
    if (!gSwitch(344)) {
        if (this.isCurrentBgm(bgm)) {
            this.updateBgmParameters(bgm);
        } else {
            this.stopBgm();
            if (bgm.name) {
                if (Decrypter.hasEncryptedAudio && this.shouldUseHtml5Audio()) {
                    this.playEncryptedBgm(bgm, pos);
                }
                else {
                    this._bgmBuffer = this.createBuffer('bgm', bgm.name);
                    this.updateBgmParameters(bgm);
                    if (!this._meBuffer) {
                        this._bgmBuffer.play(true, pos || 0);
                    }
                }
            }
        }
        this.updateCurrentBgm(bgm, pos);
    }
};

//----------------------------------------

Sprite_Timer.prototype.createBitmap = function() {
    this.bitmap = new Bitmap(160, 144);
    this.bitmap.fontSize = 32;
    
    var sfont = VictorEngine.SFont.getSFont(66);
	if (sfont) this.bitmap.sfont = sfont;
};

Sprite_Timer.prototype.redraw = function() {
    var text = this.timerText();
    var width = this.bitmap.width;
    var height = this.bitmap.height;
    this.bitmap.clear();
    this.bitmap.drawText(text, 0, -48, width, height, 'center');
};

Game_Timer.prototype.update = function(sceneActive) {
  if (!sceneActive) return;
  if (!this._working) return;
  if (this._paused) return;
  if (this._frames <= 0) return;
    
  this._frames += this._direction;
  sVar(71,this._frames);
  updateTimerBar();
    
  if (this._frames <= 0){
      this.onExpire();
      this._working = false;
      //eraseTimerImg();

      // Take dmg
      editHits(1);
      traitsFX(1,98);
      sSwitch(28,true); // HUD
      sSwitch(30,false); // Don't queue hurt animation
  }
};

// NEW FUNCTION
Game_Timer.prototype.setFrames = function(value) {
    this._frames = this._frames || 0;
    this._frames = value;
    this._working = true;
};

// NEW FUNCTION
Game_Timer.prototype.getFrames = function() {
    return this._frames;
};

Game_Timer.prototype.start = function(count) {
  Yanfly.Timer.Game_Timer_start.call(this, count);
  
  sVar(72,this._frames);
  sVar(71,this._frames);
  
  drawTimerBar();
    
  this._paused = false;
};

Game_Timer.prototype.stop = function() {
  Yanfly.Timer.Game_Timer_stop.call(this);
  this._paused = false;
  eraseTimerImg();
};

function eraseTimerImg(){
    $gameScreen.erasePicture(71);
}

function drawTimerBar(){
    $gameScreen.showPicture(71, 'red_full', 0, 5, 10, 0, 100, 255, 0);
}

function updateTimerBar(){
    var infl = gVar(71);
    var max = gVar(72);
    var percent = Math.floor((infl*-100)/max) + 100;
    $gameScreen.movePicture(71, 0, 5, 10, percent, 100, 255, 0, 20);
}

//----------------------------------------
// Fixing Gab Window

Game_Player.prototype.reserveTransfer = function(mapId, x, y, d, fadeType) {
    this._transferring = true;
    this._newMapId = mapId;
    this._newX = x;
    this._newY = y;
    this._newDirection = d;
    this._fadeType = fadeType;

    if (SceneManager._scene._gabWindow) {
        SceneManager._scene._gabWindow.storeGabs();
    }
};

Window_Gab.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    if (this.isHideGabWindow()) {
      this.hide();
    } else{
        
        if (this.visible == false) this.show();

        if (this._gabLoaded) {
            if (this._graphicLoading && this._graphicBitmap.width <= 0) return;
            this.refresh();
        } else if (this._showCount > 0) {

            if (this._showCount > 4) {
                this.updateFadeIn();
            } else {
                this.updateFadeOut();
            }

            --this._showCount;

        } else if (this._gabQueue.length > 0) {
            this.processNewGabData();
        } else if (this._gabRunning == true) {
            sSwitch(6, true); // Call HUD
            this._gabRunning = false;
        }
    }
};

Window_Gab.prototype.updateFadeIn = function() {
    this.contentsOpacity += 65;
};

Window_Gab.prototype.updateFadeOut = function() {
    this.contentsOpacity -= 65;
    /*if (this.contentsOpacity > 0) return;
    if (this._gabSwitchedOn) return;
    if (this._gabSwitch) {
      $gameSwitches.setValue(this._gabSwitch, true);
      this._gabSwitchedOn = true;
    }*/
};

Window_Gab.prototype.drawGabText = function() {
    var text = this._text;
    var wx = 12;
    var wy = 0;
    this.drawTextEx(text, wx, wy);
};

Window_Gab.prototype.storeMapGabs = function() {
    $gameTemp._storedMapGabs = this._gabQueue.slice();
    if (this.contentsOpacity > 0) {
      $gameTemp._currentMapGab = this._currentGab.slice();
      $gameTemp._currentMapGabCount = this._showCount;
    } else {
      $gameTemp._currentMapGab = [];
      $gameTemp._currentMapGabCount = undefined;
    }
};

Window_Gab.prototype.restoreMapGabs = function() {
    if ($gameTemp._storedMapGabs) {
      this._gabQueue = $gameTemp._storedMapGabs;
      $gameTemp._storedMapGabs = undefined;
    }
    if ($gameTemp._currentMapGab && $gameTemp._currentMapGab.length > 0) {
      this._gabQueue.unshift($gameTemp._currentMapGab);
      $gameTemp._currentMapGab = undefined;
    }
};

Window_Gab.prototype.startCountdown = function() {
    this._graphicLoading = false;
    this.contentsOpacity = 255;
    this._showCount = Yanfly.Param.GabBaseTime;
    var text = this._text.replace(/\\(.*?)\[(.*?)\]/gi, '');
    this._showCount += text.length * Yanfly.Param.GabTimePerChar;
    this._showCountTotal = this._showCount;

    if ($gameTemp._currentMapGabCount) {
        this._showCount = $gameTemp._currentMapGabCount;
        $gameTemp._currentMapGabCount = undefined;
    }
};

Window_Gab.prototype.setupLoadGraphic = function() {
    this._graphicLoading = false;
    if (this._graphicType === 'character') {
      this._graphicBitmap = ImageManager.loadCharacter(this._graphicName);
      this._graphicLoading = true;
    } else if (this._graphicType === 'face') {
      this._graphicBitmap = ImageManager.loadFace(this._graphicName);
      this._graphicLoading = true;
    }
    sSwitch(10, true); // Call HUD Erase
};

Window_Gab.prototype.isHideGabWindow = function() {
    if (gSwitch(5) || gSwitch(13)) return true;
    //if ($gameMessage._text && $gameMessage._text.length > 0) return true;
    if($gameMessage.isBusy()) return true;
    if ($gameParty.inBattle() && BattleManager._victoryPhase) return true;
    return false;
};

//----------------------------------------

// Remove MenuScene Bg
SceneManager.snapForBackground = function() {};

//----------------------------------------

Scene_Map.prototype.isFastForward = function() {
    return false;
};

//---------------------------------------